#include <aocpp/Startup.hpp>

#include <cstdlib>
#include <iostream>
#include <fstream>
#include <utility>

#include <unistd.h>

#define DOCTEST_CONFIG_IMPLEMENT
#include <doctest.h>

auto main(int argc, char ** argv) -> int
{
  bool run_tests {};
  char const* in_name {};
  char const* out_name {};

  try {
    int ch;
    while ((ch = getopt(argc, argv, ":hi:o:t")) != -1) {
      switch (ch) {
        case 'i': in_name = optarg; break;
        case 'o': out_name = optarg; break;
        case 't': run_tests = true; break;
        case ':':
          std::cerr << "Missing argument to -" << char(optopt) << std::endl;
          return 1;
        case 'h':
          std::cerr << "Usage: " << argv[0] << " [-i INPUT] [-o OUTPUT] [-t]" << std::endl;
          return 0;
        case '?':
        default:
          std::cerr << "Unknown flag -" << char(optopt) << std::endl;
          return 1;
      }
    }

    if (run_tests) {
      return doctest::Context{argc, argv}.run();
    }

    std::ifstream fin;
    std::ofstream fout;
    std::istream & in  {in_name  ? fin  = std::ifstream{in_name } : std::cin };
    std::ostream & out {out_name ? fout = std::ofstream{out_name} : std::cout};

    if (in.fail()) {
      std::cerr << "Bad input file" << std::endl;
      return 1;
    }

    if (out.fail()) {
      std::cerr << "Bad output file" << std::endl;
      return 1;
    }

    Main(in, out);
  } catch (std::exception const& e) {
    std::cerr << "Program failed: " << e.what() << std::endl;
    return 1;
  }
}
